@@ -8,6 +8,8 @@
 #include <stdio.h>
 #include <string.h>
 
+#include "ruby.h"
+#include "ruby/encoding.h"
 #include "ox.h"
 #include "buf.h"
 #include "err.h"
@@ -17,7 +19,7 @@
 typedef struct _element {
     char	*name;
     char	buf[64];
-    int		len;
+    long	len;
     bool	has_child;
     bool	non_text_child;
 } *Element;
@@ -124,7 +126,7 @@ append_string(Builder b, const char *str, size_t size, const char *table, bool s
 	char	buf[256];
 	char	*end = buf + sizeof(buf) - 1;
 	char	*bp = buf;
-	int	i = size;
+	size_t	i = size;
 	int	fcnt;
 
 	for (; '\0' != *str && 0 < i; i--, str++) {
@@ -183,7 +185,7 @@ append_string(Builder b, const char *str, size_t size, const char *table, bool s
 static void
 append_sym_str(Builder b, VALUE v) {
     const char	*s;
-    int		len;
+    long	len;
 
     switch (rb_type(v)) {
     case T_STRING:
@@ -219,7 +221,9 @@ i_am_a_child(Builder b, bool is_text) {
 }
 
 static int
-append_attr(VALUE key, VALUE value, Builder b) {
+append_attr(VALUE key, VALUE value, VALUE bv) {
+    Builder	b = (Builder)bv;
+
     buf_append(&b->buf, ' ');
     b->col++;
     b->pos++;
@@ -280,7 +284,7 @@ pop(Builder b) {
 	    append_indent(b);
 	}
 	buf_append_string(&b->buf, "</", 2);
-	buf_append_string(&b->buf, e->name, e->len);
+	append_string(b, e->name, e->len, xml_element_chars, false);
 	buf_append(&b->buf, '>');
 	b->col += e->len + 3;
 	b->pos += e->len + 3;
@@ -329,9 +333,7 @@ to_s(Builder b) {
     rstr = rb_str_new(b->buf.head, buf_len(&b->buf));
 
     if ('\0' != *b->encoding) {
-#if HAS_ENCODING_SUPPORT
 	rb_enc_associate(rstr, rb_enc_find(b->encoding));
-#endif
     }
     return rstr;
 }
@@ -600,7 +602,7 @@ builder_element(int argc, VALUE *argv, VALUE self) {
     Builder		b = (Builder)DATA_PTR(self);
     Element		e;
     const char		*name;
-    int			len;
+    long		len;
 
     if (1 > argc) {
 	rb_raise(ox_arg_error_class, "missing element name");
@@ -662,7 +664,7 @@ static VALUE
 builder_void_element(int argc, VALUE *argv, VALUE self) {
     Builder	b = (Builder)DATA_PTR(self);
     const char	*name;
-    int		len;
+    long	len;
 
     if (1 > argc) {
 	rb_raise(ox_arg_error_class, "missing element name");
@@ -892,8 +894,7 @@ builder_pos(VALUE self) {
  *
  * Closes the current element.
  */
-static VALUE
-builder_pop(VALUE self) {
+static VALUE builder_pop(VALUE self) {
     pop((Builder)DATA_PTR(self));
 
     return Qnil;
@@ -915,7 +916,12 @@ builder_close(VALUE self) {
  *
  * An XML builder.
  */
-void ox_init_builder(VALUE ox) {
+void
+ox_init_builder(VALUE ox) {
+#if 0
+    // Just for rdoc.
+    ox = rb_define_module("Ox");
+#endif
     builder_class = rb_define_class_under(ox, "Builder", rb_cObject);
     rb_define_module_function(builder_class, "new", builder_new, -1);
     rb_define_module_function(builder_class, "file", builder_file, -1);
