@@ -18,11 +18,11 @@
 #define DEPTH		16
 
 typedef union {
-    struct _Cache8	*child;
+    struct _cache8	*child;
     slot_t		value;
 } Bucket;
 
-struct _Cache8 {
+struct _cache8 {
     Bucket	buckets[SLOT_CNT];
 };
 
@@ -33,8 +33,8 @@ void
 ox_cache8_new(Cache8 *cache) {
     Bucket	*b;
     int		i;
-    
-    *cache = ALLOC(struct _Cache8);
+
+    *cache = ALLOC(struct _cache8);
     for (i = SLOT_CNT, b = (*cache)->buckets; 0 < i; i--, b++) {
 	b->value = 0;
     }
@@ -66,7 +66,7 @@ ox_cache8_get(Cache8 cache, sid_t key, slot_t **slot) {
     int		i;
     sid_t	k8 = (sid_t)key;
     sid_t	k;
-    
+
     for (i = 64 - BITS; 0 < i; i -= BITS) {
 	k = (k8 >> i) & MASK;
 	b = cache->buckets + k;
