@@ -54,7 +54,7 @@ static void	dump_obj(ID aid, VALUE obj, int depth, Out out);
 static void	dump_gen_doc(VALUE obj, int depth, Out out);
 static void	dump_gen_element(VALUE obj, int depth, Out out);
 static void	dump_gen_instruct(VALUE obj, int depth, Out out);
-static int	dump_gen_attr(VALUE key, VALUE value, Out out);
+static int	dump_gen_attr(VALUE key, VALUE value, VALUE ov);
 static int	dump_gen_nodes(VALUE obj, int depth, Out out);
 static void	dump_gen_val_node(VALUE obj, int depth,
 				  const char *pre, size_t plen,
@@ -67,12 +67,12 @@ static void	grow(Out out, size_t len);
 
 static void	dump_value(Out out, const char *value, size_t size);
 static void	dump_str_value(Out out, const char *value, size_t size, const char *table);
-static int	dump_var(ID key, VALUE value, Out out);
+static int	dump_var(ID key, VALUE value, VALUE ov);
 static void	dump_num(Out out, VALUE obj);
 static void	dump_date(Out out, VALUE obj);
 static void	dump_time_thin(Out out, VALUE obj);
 static void	dump_time_xsd(Out out, VALUE obj);
-static int	dump_hash(VALUE key, VALUE value, Out out);
+static int	dump_hash(VALUE key, VALUE value, VALUE ov);
 
 static int	is_xml_friendly(const uchar *str, int len, const char *table);
 
@@ -301,7 +301,14 @@ dump_start(Out out, Element e) {
 	fill_attr(out, 'i', s, end - s);
     }
     if (e->closed) {
-	*out->cur++ = '/';
+	if (out->opts->no_empty) {
+	    *out->cur++ = '>';
+	    *out->cur++ = '<';
+	    *out->cur++ = '/';
+	    *out->cur++ = e->type;
+	} else {
+	    *out->cur++ = '/';
+	}
     }
     *out->cur++ = '>';
     *out->cur = '\0';
@@ -448,17 +455,14 @@ static void
 dump_time_thin(Out out, VALUE obj) {
     char		buf[64];
     char		*b = buf + sizeof(buf) - 1;
-#if HAS_RB_TIME_TIMESPEC
+#if HAVE_RB_TIME_TIMESPEC
     struct timespec	ts = rb_time_timespec(obj);
     time_t		sec = ts.tv_sec;
     long		nsec = ts.tv_nsec;
 #else
     time_t		sec = NUM2LONG(rb_funcall2(obj, ox_tv_sec_id, 0, 0));
-#if HAS_NANO_TIME
     long		nsec = NUM2LONG(rb_funcall2(obj, ox_tv_nsec_id, 0, 0));
-#else
-    long		nsec = NUM2LONG(rb_funcall2(obj, ox_tv_usec_id, 0, 0)) * 1000;
-#endif
+    //long		nsec = NUM2LONG(rb_funcall2(obj, ox_tv_usec_id, 0, 0)) * 1000;
 #endif
     char		*dot = b - 10;
     long		size;
@@ -507,17 +511,14 @@ dump_date(Out out, VALUE obj) {
 static void
 dump_time_xsd(Out out, VALUE obj) {
     struct tm		*tm;
-#if HAS_RB_TIME_TIMESPEC
+#if HAVE_RB_TIME_TIMESPEC
     struct timespec	ts = rb_time_timespec(obj);
     time_t		sec = ts.tv_sec;
     long		nsec = ts.tv_nsec;
 #else
     time_t		sec = NUM2LONG(rb_funcall2(obj, ox_tv_sec_id, 0, 0));
-#if HAS_NANO_TIME
     long		nsec = NUM2LONG(rb_funcall2(obj, ox_tv_nsec_id, 0, 0));
-#else
-    long		nsec = NUM2LONG(rb_funcall2(obj, ox_tv_usec_id, 0, 0)) * 1000;
-#endif
+    //long		nsec = NUM2LONG(rb_funcall2(obj, ox_tv_usec_id, 0, 0)) * 1000;
 #endif
     int			tzhour, tzmin;
     char		tzsign = '+';
@@ -527,7 +528,7 @@ dump_time_xsd(Out out, VALUE obj) {
     }
     /* 2010-07-09T10:47:45.895826+09:00 */
     tm = localtime(&sec);
-#if HAS_TM_GMTOFF
+#if HAVE_ST_TM_GMTOFF
     if (0 > tm->tm_gmtoff) {
 	tzsign = '-';
 	tzhour = (int)(tm->tm_gmtoff / -3600);
@@ -802,7 +803,7 @@ dump_obj(ID aid, VALUE obj, int depth, Out out) {
     }
     case T_STRUCT:
     {
-#if HAS_RSTRUCT
+#ifdef RSTRUCT_GET
 	VALUE	clas;
 
 	if (0 != out->circ_cache && check_circular(out, obj, &e)) {
@@ -824,7 +825,7 @@ dump_obj(ID aid, VALUE obj, int depth, Out out) {
 	} else {
 	    char	num_buf[16];
 	    int		d2 = depth + 1;
-#if UNIFY_FIXNUM_AND_BIGNUM
+#ifdef RUBY_INTEGER_UNIFICATION
 		long i;
 		long cnt = NUM2LONG(rb_struct_size(obj));
 #else // UNIFY_FIXNUM_AND_INTEGER
@@ -837,7 +838,7 @@ dump_obj(ID aid, VALUE obj, int depth, Out out) {
 	    out->w_start(out, &e);
 
 	    for (i = 0; i < cnt; i++) {
-	        VALUE v = RSTRUCT_GET(obj, i);
+	        VALUE v = RSTRUCT_GET(obj, (int)(i));
 	        dump_obj(rb_intern(ulong2str(i, num_buf + sizeof(num_buf) - 1)), v, d2, out);
 	    }
 	    out->w_end(out, &e);
@@ -870,7 +871,7 @@ dump_obj(ID aid, VALUE obj, int depth, Out out) {
 	    dump_gen_element(obj, depth + 1, out);
 	    out->w_end(out, &e);
 	} else { /* Object */
-#if HAS_IVAR_HELPERS
+#if HAVE_RB_IVAR_FOREACH
 	    e.type = (Qtrue == rb_obj_is_kind_of(obj, rb_eException)) ? ExceptionCode : ObjectCode;
 	    cnt = (int)rb_ivar_count(obj);
 	    e.closed = (0 >= cnt);
@@ -998,7 +999,9 @@ dump_obj(ID aid, VALUE obj, int depth, Out out) {
 }
 
 static int
-dump_var(ID key, VALUE value, Out out) {
+dump_var(ID key, VALUE value, VALUE ov) {
+    Out	out = (Out)ov;
+
     if (T_DATA == rb_type(value) && key == ox_mesg_id) {
 	/* There is a secret recipe that keeps Exception mesg attributes as a
 	 * T_DATA until it is needed. The safe way around this hack is to call
@@ -1015,7 +1018,9 @@ dump_var(ID key, VALUE value, Out out) {
 }
 
 static int
-dump_hash(VALUE key, VALUE value, Out out) {
+dump_hash(VALUE key, VALUE value, VALUE ov) {
+    Out	out = (Out)ov;
+
     dump_obj(0, key, out->depth, out);
     dump_obj(0, value, out->depth, out);
 
@@ -1105,6 +1110,11 @@ dump_gen_element(VALUE obj, int depth, Out out) {
 	*out->cur++ = '<';
 	*out->cur++ = '/';
 	fill_value(out, name, nlen);
+    } else if (out->opts->no_empty) {
+	*out->cur++ = '>';
+	*out->cur++ = '<';
+	*out->cur++ = '/';
+	fill_value(out, name, nlen);
     } else {
 	*out->cur++ = '/';
     }
@@ -1186,16 +1196,13 @@ dump_gen_nodes(VALUE obj, int depth, Out out) {
 }
 
 static int
-dump_gen_attr(VALUE key, VALUE value, Out out) {
+dump_gen_attr(VALUE key, VALUE value, VALUE ov) {
+    Out	out = (Out)ov;
+
     const char	*ks;
     size_t	klen;
     size_t	size;
 
-#if HAS_PRIVATE_ENCODING
-    // There seems to be a bug in jruby for converting symbols to strings and preserving the encoding. This is a work
-    // around.
-    ks = rb_str_ptr(rb_String(key));
-#else
     switch (rb_type(key)) {
     case T_SYMBOL:
 	ks = rb_id2name(SYM2ID(key));
@@ -1208,7 +1215,6 @@ dump_gen_attr(VALUE key, VALUE value, Out out) {
 	ks = StringValuePtr(key);
 	break;
     }
-#endif
     klen = strlen(ks);
     value = rb_String(value);
     size = 4 + klen + RSTRING_LEN(value);
