@@ -11,8 +11,11 @@
 #include <time.h>
 
 #include "ruby.h"
+#include "ruby/encoding.h"
+
 #include "base64.h"
 #include "ox.h"
+#include "intern.h"
 
 static void	instruct(PInfo pi, const char *target, Attr attrs, const char *content);
 static void	add_text(PInfo pi, char *text, int closed);
@@ -39,10 +42,10 @@ static void		fill_indent(PInfo pi, char *buf, size_t size);
 
 
 struct _parseCallbacks	 _ox_obj_callbacks = {
-    instruct, /* instruct, */
-    0, /* add_doctype, */
-    0, /* add_comment, */
-    0, /* add_cdata, */
+    instruct,	// instruct,
+    0,		// add_doctype,
+    0,		// add_comment,
+    0,		// add_cdata,
     add_text,
     add_element,
     end_element,
@@ -53,56 +56,6 @@ ParseCallbacks	 ox_obj_callbacks = &_ox_obj_callbacks;
 
 extern ParseCallbacks	ox_gen_callbacks;
 
-
-inline static VALUE
-str2sym(const char *str, void *encoding) {
-    VALUE	sym;
-
-#ifdef HAVE_RUBY_ENCODING_H
-    if (0 != encoding) {
-	VALUE	rstr = rb_str_new2(str);
-
-	rb_enc_associate(rstr, (rb_encoding*)encoding);
-	sym = rb_funcall(rstr, ox_to_sym_id, 0);
-    } else {
-	sym = ID2SYM(rb_intern(str));
-    }
-#else
-    sym = ID2SYM(rb_intern(str));
-#endif
-    return sym;
-}
-
-inline static ID
-name2var(const char *name, void *encoding) {
-    VALUE	*slot;
-    ID		var_id = 0;
-
-    if ('0' <= *name && *name <= '9') {
-	var_id = INT2NUM(atoi(name));
-    } else if (Qundef == (var_id = ox_cache_get(ox_attr_cache, name, &slot, 0))) {
-#ifdef HAVE_RUBY_ENCODING_H
-	if (0 != encoding) {
-	    volatile VALUE	rstr = rb_str_new2(name);
-	    volatile VALUE	sym;
-
-	    rb_enc_associate(rstr, (rb_encoding*)encoding);
-	    sym = rb_funcall(rstr, ox_to_sym_id, 0);
-	    // Needed for Ruby 2.2 to get around the GC of symbols
-	    // created with to_sym which is needed for encoded symbols.
-	    rb_ary_push(ox_sym_bank, sym);
-	    var_id = SYM2ID(sym);
-	} else {
-	    var_id = rb_intern(name);
-	}
-#else
-	var_id = rb_intern(name);
-#endif
-	*slot = var_id;
-    }
-    return var_id;
-}
-
 inline static VALUE
 resolve_classname(VALUE mod, const char *class_name, Effort effort, VALUE base_class) {
     VALUE	clas;
@@ -125,7 +78,7 @@ resolve_classname(VALUE mod, const char *class_name, Effort effort, VALUE base_c
 	break;
     case StrictEffort:
     default:
-	/* raise an error if name is not defined */
+	// raise an error if name is not defined
 	clas = rb_const_get_at(mod, ci);
 	break;
     }
@@ -143,7 +96,6 @@ classname2obj(const char *name, PInfo pi, VALUE base_class) {
     }
 }
 
-#if HAS_RSTRUCT
 inline static VALUE
 structname2obj(const char *name) {
     VALUE	ost;
@@ -159,16 +111,12 @@ structname2obj(const char *name) {
 	}
     }
     ost = rb_const_get(ox_struct_class, rb_intern(s));
-    /* use encoding as the indicator for Ruby 1.8.7 or 1.9.x */
-#if HAS_ENCODING_SUPPORT
-    return rb_struct_alloc_noinit(ost);
-#elif HAS_PRIVATE_ENCODING
+#if HAVE_RB_STRUCT_ALLOC_NOINIT
     return rb_struct_alloc_noinit(ost);
 #else
     return rb_struct_new(ost);
 #endif
 }
-#endif
 
 inline static VALUE
 parse_ulong(const char *s, PInfo pi) {
@@ -185,7 +133,7 @@ parse_ulong(const char *s, PInfo pi) {
     return ULONG2NUM(n);
 }
 
-/* 2010-07-09T10:47:45.895826162+09:00 */
+// 2010-07-09T10:47:45.895826162+09:00
 inline static VALUE
 parse_time(const char *text, VALUE clas) {
     VALUE	t;
@@ -205,7 +153,7 @@ classname2class(const char *name, PInfo pi, VALUE base_class) {
     VALUE	*slot;
     VALUE	clas;
 
-    if (Qundef == (clas = ox_cache_get(ox_class_cache, name, &slot, 0))) {
+    if (Qundef == (clas = slot_cache_get(ox_class_cache, name, &slot, 0))) {
 	char		class_name[1024];
 	char		*s;
 	const char	*n = name;
@@ -230,6 +178,7 @@ classname2class(const char *name, PInfo pi, VALUE base_class) {
 	*s = '\0';
 	if (Qundef != (clas = resolve_classname(clas, class_name, pi->options->effort, base_class))) {
 	    *slot = clas;
+	    rb_gc_register_address(slot);
 	}
     }
     return clas;
@@ -239,7 +188,12 @@ static ID
 get_var_sym_from_attrs(Attr a, void *encoding) {
     for (; 0 != a->name; a++) {
 	if ('a' == *a->name && '\0' == *(a->name + 1)) {
-	    return name2var(a->value, encoding);
+	    const char	*val = a->value;
+
+	    if ('0' <= *val && *val <= '9') {
+		return INT2NUM(atoi(val));
+	    }
+	    return ox_id_intern(val, strlen(val));
 	}
     }
     return 0;
@@ -255,7 +209,6 @@ get_obj_from_attrs(Attr a, PInfo pi, VALUE base_class) {
     return Qundef;
 }
 
-#if HAS_RSTRUCT
 static VALUE
 get_struct_from_attrs(Attr a) {
     for (; 0 != a->name; a++) {
@@ -265,7 +218,6 @@ get_struct_from_attrs(Attr a) {
     }
     return Qundef;
 }
-#endif
 
 static VALUE
 get_class_from_attrs(Attr a, PInfo pi, VALUE base_class) {
@@ -363,7 +315,7 @@ parse_regexp(const char *text) {
     int		options = 0;
 
     te = text + strlen(text) - 1;
-#if HAS_ONIG
+#ifdef ONIG_OPTION_IGNORECASE
     for (; text < te && '/' != *te; te--) {
 	switch (*te) {
 	case 'i':	options |= ONIG_OPTION_IGNORECASE;	break;
@@ -379,19 +331,11 @@ parse_regexp(const char *text) {
 static void
 instruct(PInfo pi, const char *target, Attr attrs, const char *content) {
     if (0 == strcmp("xml", target)) {
-#if HAS_ENCODING_SUPPORT
 	for (; 0 != attrs->name; attrs++) {
 	    if (0 == strcmp("encoding", attrs->name)) {
 		pi->options->rb_enc = rb_enc_find(attrs->value);
 	    }
 	}
-#elif HAS_PRIVATE_ENCODING
-	for (; 0 != attrs->name; attrs++) {
-	    if (0 == strcmp("encoding", attrs->name)) {
-		pi->options->rb_enc = rb_str_new2(attrs->value);
-	    }
-	}
-#endif
     }
 }
 
@@ -417,15 +361,9 @@ add_text(PInfo pi, char *text, int closed) {
     case NoCode:
     case StringCode:
 	h->obj = rb_str_new2(text);
-#if HAS_ENCODING_SUPPORT
 	if (0 != pi->options->rb_enc) {
 	    rb_enc_associate(h->obj, pi->options->rb_enc);
 	}
-#elif HAS_PRIVATE_ENCODING
-	if (Qnil != pi->options->rb_enc) {
-	    rb_funcall(h->obj, ox_force_encoding_id, 1, pi->options->rb_enc);
-	}
-#endif
 	if (0 != pi->circ_array) {
 	    circ_array_set(pi->circ_array, h->obj, (unsigned long)pi->id);
 	}
@@ -459,20 +397,8 @@ add_text(PInfo pi, char *text, int closed) {
 	h->obj = rb_float_new(strtod(text, 0));
 	break;
     case SymbolCode:
-    {
-	VALUE	sym;
-	VALUE	*slot;
-
-	if (Qundef == (sym = ox_cache_get(ox_symbol_cache, text, &slot, 0))) {
-	    sym = str2sym(text, (void*)pi->options->rb_enc);
-	    // Needed for Ruby 2.2 to get around the GC of symbols created with
-	    // to_sym which is needed for encoded symbols.
-	    rb_ary_push(ox_sym_bank, sym);
-	    *slot = sym;
-	}
-	h->obj = sym;
+	h->obj = ox_sym_intern(text, strlen(text), NULL);
 	break;
-    }
     case DateCode:
     {
 	VALUE	args[1];
@@ -494,15 +420,9 @@ add_text(PInfo pi, char *text, int closed) {
 
 	from_base64(text, (uchar*)str);
 	v = rb_str_new(str, str_size);
-#if HAS_ENCODING_SUPPORT
 	if (0 != pi->options->rb_enc) {
 	    rb_enc_associate(v, pi->options->rb_enc);
 	}
-#elif HAS_PRIVATE_ENCODING
-	if (0 != pi->options->rb_enc) {
-	    rb_funcall(v, ox_force_encoding_id, 1, pi->options->rb_enc);
-	}
-#endif
 	if (0 != pi->circ_array) {
 	    circ_array_set(pi->circ_array, v, (unsigned long)h->obj);
 	}
@@ -511,20 +431,11 @@ add_text(PInfo pi, char *text, int closed) {
     }
     case Symbol64Code:
     {
-	VALUE		sym;
-	VALUE		*slot;
 	unsigned long	str_size = b64_orig_size(text);
 	char		*str = ALLOCA_N(char, str_size + 1);
 
 	from_base64(text, (uchar*)str);
-	if (Qundef == (sym = ox_cache_get(ox_symbol_cache, str, &slot, 0))) {
-	    sym = str2sym(str, (void*)pi->options->rb_enc);
-	    // Needed for Ruby 2.2 to get around the GC of symbols created with
-	    // to_sym which is needed for encoded symbols.
-	    rb_ary_push(ox_sym_bank, sym);
-	    *slot = sym;
-	}
-	h->obj = sym;
+	h->obj = ox_sym_intern(str, strlen(str), NULL);
 	break;
     }
     case RegexpCode:
@@ -542,11 +453,7 @@ add_text(PInfo pi, char *text, int closed) {
 	h->obj = rb_cstr_to_inum(text, 10, 1);
 	break;
     case BigDecimalCode:
-#if HAS_BIGDECIMAL
 	h->obj = rb_funcall(rb_cObject, ox_bigdecimal_id, 1, rb_str_new2(text));
-#else
-	h->obj = Qnil;
-#endif
 	break;
     default:
 	h->obj = Qnil;
@@ -580,7 +487,7 @@ add_element(PInfo pi, const char *ename, Attr attrs, int hasChildren) {
 	    printf("%s%s\n", indent, buf);
 	}
     }
-    if (helper_stack_empty(&pi->helpers)) { /* top level object */
+    if (helper_stack_empty(&pi->helpers)) { // top level object
 	if (0 != (id = get_id_from_attrs(pi, attrs))) {
 	    pi->circ_array = circ_array_new();
 	}
@@ -601,7 +508,7 @@ add_element(PInfo pi, const char *ename, Attr attrs, int hasChildren) {
 	h->obj = Qfalse;
 	break;
     case StringCode:
-	/* h->obj will be replaced by add_text if it is called */
+	// h->obj will be replaced by add_text if it is called
 	h->obj = ox_empty_string;
 	if (0 != pi->circ_array) {
 	    pi->id = get_id_from_attrs(pi, attrs);
@@ -618,8 +525,8 @@ add_element(PInfo pi, const char *ename, Attr attrs, int hasChildren) {
     case ComplexCode:
     case DateCode:
     case TimeCode:
-    case RationalCode: /* sub elements read next */
-	/* value will be read in the following add_text */
+    case RationalCode: // sub elements read next
+	// value will be read in the following add_text
 	h->obj = Qundef;
 	break;
     case String64Code:
@@ -641,7 +548,7 @@ add_element(PInfo pi, const char *ename, Attr attrs, int hasChildren) {
 	}
 	break;
     case RangeCode:
-	h->obj = rb_range_new(ox_zero_fixnum, ox_zero_fixnum, Qfalse);
+	h->obj = rb_ary_new_from_args(3, Qnil, Qnil, Qfalse);
 	break;
     case RawCode:
 	if (hasChildren) {
@@ -670,15 +577,10 @@ add_element(PInfo pi, const char *ename, Attr attrs, int hasChildren) {
 	}
 	break;
     case StructCode:
-#if HAS_RSTRUCT
 	h->obj = get_struct_from_attrs(attrs);
 	if (0 != pi->circ_array) {
 	    circ_array_set(pi->circ_array, h->obj, get_id_from_attrs(pi, attrs));
 	}
-#else
-	set_error(&pi->err, "Ruby structs not supported with this verion of Ruby", pi->str, pi->s);
-	return;
-#endif
 	break;
     case ClassCode:
 	if (Qundef == (h->obj = get_class_from_attrs(attrs, pi, ox_bag_clas))) {
@@ -726,12 +628,15 @@ end_element(PInfo pi, const char *ename) {
 	Helper	ph = helper_stack_peek(&pi->helpers);
 
 	if (ox_empty_string == h->obj) {
-	    /* special catch for empty strings */
+	    // special catch for empty strings
 	    h->obj = rb_str_new2("");
-	}
-	if (Qundef == h->obj) {
+	} else if (Qundef == h->obj) {
 	    set_error(&pi->err, "Invalid element for object mode", pi->str, pi->s);
 	    return;
+	} else if (RangeCode == h->type) { // Expect an array of 3 elements.
+	    const VALUE	*ap = RARRAY_PTR(h->obj);
+
+	    h->obj = rb_range_new(*ap, *(ap + 1), Qtrue == *(ap + 2));
 	}
 	pi->obj = h->obj;
 	if (0 != ph) {
@@ -742,45 +647,39 @@ end_element(PInfo pi, const char *ename) {
 	    case ExceptionCode:
 	    case ObjectCode:
 		if (Qnil != ph->obj) {
-		    if (0 == h->var) {
+		    if (0 == h->var || NULL == rb_id2name(h->var )) {
 			set_error(&pi->err, "Invalid element for object mode", pi->str, pi->s);
 			return;
 		    }
+		    if (RUBY_T_OBJECT != rb_type(ph->obj)) {
+			set_error(&pi->err, "Corrupt object encoding", pi->str, pi->s);
+			return;
+		    }
 		    rb_ivar_set(ph->obj, h->var, h->obj);
 		}
 		break;
 	    case StructCode:
-#if HAS_RSTRUCT
 		if (0 == h->var) {
 		    set_error(&pi->err, "Invalid element for object mode", pi->str, pi->s);
 		    return;
 		}
 		rb_struct_aset(ph->obj, h->var, h->obj);
-#else
-		set_error(&pi->err, "Ruby structs not supported with this verion of Ruby", pi->str, pi->s);
-		return;
-#endif
 		break;
 	    case HashCode:
 		// put back h
 		helper_stack_push(&pi->helpers, h->var, h->obj, KeyCode);
 		break;
 	    case RangeCode:
-#if HAS_RSTRUCT
 		if (ox_beg_id == h->var) {
-		    RSTRUCT_SET(ph->obj, 0, h->obj);
+		    rb_ary_store(ph->obj, 0, h->obj);
 		} else if (ox_end_id == h->var) {
-		    RSTRUCT_SET(ph->obj, 1, h->obj);
+		    rb_ary_store(ph->obj, 1, h->obj);
 		} else if (ox_excl_id == h->var) {
-		    RSTRUCT_SET(ph->obj, 2, h->obj);
+		    rb_ary_store(ph->obj, 2, h->obj);
 		} else {
 		    set_error(&pi->err, "Invalid range attribute", pi->str, pi->s);
 		    return;
 		}
-#else
-		set_error(&pi->err, "Ruby structs not supported with this verion of Ruby", pi->str, pi->s);
-		return;
-#endif
 		break;
 	    case KeyCode:
 		{
@@ -872,7 +771,7 @@ parse_double_time(const char *text, VALUE clas) {
     for (; text - dot <= 9; text++) {
 	v2 *= 10;
     }
-#if HAS_NANO_TIME
+#if HAVE_RB_TIME_NANO_NEW
     return rb_time_nano_new(v, v2);
 #else
     return rb_time_new(v, v2 / 1000);
@@ -928,14 +827,14 @@ parse_xsd_time(const char *text, VALUE clas) {
     tm.tm_hour = (int)cargs[3];
     tm.tm_min = (int)cargs[4];
     tm.tm_sec = (int)cargs[5];
-#if HAS_NANO_TIME
+#if HAVE_RB_TIME_NANO_NEW
     return rb_time_nano_new(mktime(&tm), cargs[6]);
 #else
     return rb_time_new(mktime(&tm), cargs[6] / 1000);
 #endif
 }
 
-/* debug functions */
+// debug functions
 static void
 fill_indent(PInfo pi, char *buf, size_t size) {
     size_t	cnt;
