@@ -5,13 +5,16 @@
 
 #include <stdlib.h>
 #include <errno.h>
+#include <stdbool.h>
 #include <stdio.h>
 #include <string.h>
+#include <strings.h>
 
 #include "ruby.h"
 #include "ox.h"
 #include "err.h"
 #include "attr.h"
+#include "intern.h"
 #include "helper.h"
 #include "special.h"
 
@@ -93,6 +96,30 @@ next_white(PInfo pi) {
     }
 }
 
+static void fix_newlines(char *buf) {
+#if HAVE_INDEX
+    if (NULL != index(buf, '\r')) {
+#endif
+	char	*s = buf;
+	char	*d = buf;
+
+	for (; '\0' != *s; s++) {
+	    if ('\r' == *s) {
+		if ('\n' == *(s + 1)) {
+		    continue;
+		}
+		*d = '\n';
+	    } else if (d < s) {
+		*d = *s;
+	    }
+	    d++;
+	}
+	*d = '\0';
+#if HAVE_INDEX
+    }
+#endif
+}
+
 static void
 mark_pi_cb(void *ptr) {
     if (NULL != ptr) {
@@ -121,7 +148,7 @@ ox_parse(char *xml, size_t len, ParseCallbacks pcb, char **endp, Options options
     if (DEBUG <= options->trace) {
 	printf("Parsing xml:\n%s\n", xml);
     }
-    /* initialize parse info */
+    // initialize parse info
     helper_stack_init(&pi.helpers);
     // Protect against GC
     wrap = Data_Wrap_Struct(rb_cObject, mark_pi_cb, NULL, &pi);
@@ -134,8 +161,11 @@ ox_parse(char *xml, size_t len, ParseCallbacks pcb, char **endp, Options options
     pi.obj = Qnil;
     pi.circ_array = 0;
     pi.options = options;
+    pi.marked = NULL;
+    pi.mark_size = 0;
+    pi.mark_cnt = 0;
     while (1) {
-	next_non_white(&pi);	/* skip white space */
+	next_non_white(&pi);	// skip white space
 	if ('\0' == *pi.s) {
 	    break;
 	}
@@ -143,31 +173,31 @@ ox_parse(char *xml, size_t len, ParseCallbacks pcb, char **endp, Options options
 	    *endp = pi.s;
 	    break;
 	}
-	if ('<' != *pi.s) {		/* all top level entities start with < */
+	if ('<' != *pi.s) {		// all top level entities start with <
 	    set_error(err, "invalid format, expected <", pi.str, pi.s);
 	    helper_stack_cleanup(&pi.helpers);
 	    return Qnil;
 	}
-	pi.s++;		/* past < */
+	pi.s++;		// past <
 	switch (*pi.s) {
-	case '?':	/* processing instruction */
+	case '?':	// processing instruction
 	    pi.s++;
 	    read_instruction(&pi);
 	    break;
-	case '!':	/* comment or doctype */
+	case '!':	// comment or doctype
 	    pi.s++;
 	    if ('\0' == *pi.s) {
 		set_error(err, "invalid format, DOCTYPE or comment not terminated", pi.str, pi.s);
 		helper_stack_cleanup(&pi.helpers);
 		return Qnil;
 	    } else if ('-' == *pi.s) {
-		pi.s++;	/* skip - */
+		pi.s++;	// skip -
 		if ('-' != *pi.s) {
 		    set_error(err, "invalid format, bad comment format", pi.str, pi.s);
 		    helper_stack_cleanup(&pi.helpers);
 		    return Qnil;
 		} else {
-		    pi.s++;	/* skip second - */
+		    pi.s++;	// skip second -
 		    read_comment(&pi);
 		}
 	    } else if ((TolerantEffort == options->effort) ? 0 == strncasecmp("DOCTYPE", pi.s, 7) : 0 == strncmp("DOCTYPE", pi.s, 7)) {
@@ -208,32 +238,11 @@ ox_parse(char *xml, size_t len, ParseCallbacks pcb, char **endp, Options options
     return pi.obj;
 }
 
-static char*
-gather_content(const char *src, char *content, size_t len) {
-    for (; 0 < len; src++, content++, len--) {
-	switch (*src) {
-	case '?':
-	    if ('>' == *(src + 1)) {
-		*content = '\0';
-		return (char*)(src + 1);
-	    }
-	    *content = *src;
-	    break;
-	case '\0':
-	    return 0;
-	default:
-	    *content = *src;
-	    break;
-	}
-    }
-    return 0;
-}
-
-/* Entered after the "<?" sequence. Ready to read the rest.
- */
+// Entered after the "<?" sequence. Ready to read the rest.
 static void
 read_instruction(PInfo pi) {
-    char		content[1024];
+    char		content[256];
+    char		*content_ptr;
     struct _attrStack	attrs;
     char		*attr_name;
     char		*attr_value;
@@ -241,7 +250,8 @@ read_instruction(PInfo pi) {
     char		*end;
     char		c;
     char		*cend;
-    int			attrs_ok = 1;
+    size_t		size;
+    bool		attrs_ok = true;
 
     *content = '\0';
     attr_stack_init(&attrs);
@@ -249,13 +259,36 @@ read_instruction(PInfo pi) {
 	return;
     }
     end = pi->s;
-    if (0 == (cend = gather_content(pi->s, content, sizeof(content) - 1))) {
-	set_error(&pi->err, "processing instruction content too large or not terminated", pi->str, pi->s);
-	return;
+    for (; true; pi->s++) {
+        switch (*pi->s) {
+        case '?':
+            if ('>' == *(pi->s + 1)) {
+		pi->s++;
+		goto DONE;
+            }
+            break;
+        case '\0':
+	    set_error(&pi->err, "processing instruction not terminated", pi->str, pi->s);
+	    return;
+        default:
+	    break;
+        }
     }
+DONE:
+    cend = pi->s;
+    size = cend - end - 1;
+    pi->s = end;
+    if (size < sizeof(content)) {
+	content_ptr = content;
+    } else {
+	content_ptr = ALLOC_N(char, size + 1);
+    }
+    memcpy(content_ptr, end, size);
+    content_ptr[size] = '\0';
+
     next_non_white(pi);
     c = *pi->s;
-    *end = '\0'; /* terminate name */
+    *end = '\0'; // terminate name
     if ('?' != c) {
 	while ('?' != c) {
 	    pi->last = 0;
@@ -272,11 +305,11 @@ read_instruction(PInfo pi) {
 	    end = pi->s;
 	    next_non_white(pi);
 	    if ('=' != *pi->s++) {
-		attrs_ok = 0;
+		attrs_ok = false;
 		break;
 	    }
-	    *end = '\0'; /* terminate name */
-	    /* read value */
+	    *end = '\0'; // terminate name
+	    // read value
 	    next_non_white(pi);
 	    if (0 == (attr_value = read_quoted_value(pi))) {
 		attr_stack_cleanup(&attrs);
@@ -309,10 +342,13 @@ read_instruction(PInfo pi) {
 	if (attrs_ok) {
 	    pi->pcb->instruct(pi, target, attrs.head, 0);
 	} else {
-	    pi->pcb->instruct(pi, target, attrs.head, content);
+	    pi->pcb->instruct(pi, target, attrs.head, content_ptr);
 	}
     }
     attr_stack_cleanup(&attrs);
+    if (content_ptr != content) {
+	xfree(content_ptr);
+    }
 }
 
 static void
@@ -355,15 +391,14 @@ read_delimited(PInfo pi, char end) {
     }
 }
 
-/* Entered after the "<!DOCTYPE" sequence plus the first character after
- * that. Ready to read the rest.
- */
+// Entered after the "<!DOCTYPE" sequence plus the first character after
+// that. Ready to read the rest.
 static void
 read_doctype(PInfo pi) {
-    char	*docType;
+    char	*doctype;
 
     next_non_white(pi);
-    docType = pi->s;
+    doctype = pi->s;
     read_delimited(pi, '>');
     if (err_has(&pi->err)) {
 	return;
@@ -372,12 +407,12 @@ read_doctype(PInfo pi) {
     *pi->s = '\0';
     pi->s++;
     if (0 != pi->pcb->add_doctype) {
-	pi->pcb->add_doctype(pi, docType);
+	fix_newlines(doctype);
+	pi->pcb->add_doctype(pi, doctype);
     }
 }
 
-/* Entered after "<!--". Returns error code.
- */
+// Entered after "<!--". Returns error code.
 static void
 read_comment(PInfo pi) {
     char	*end;
@@ -406,16 +441,16 @@ read_comment(PInfo pi) {
 	    break;
 	}
     }
-    *end = '\0'; /* in case the comment was blank */
+    *end = '\0'; // in case the comment was blank
     pi->s = end + 3;
     if (0 != pi->pcb->add_comment) {
+	fix_newlines(comment);
 	pi->pcb->add_comment(pi, comment);
     }
 }
 
-/* Entered after the '<' and the first character after that. Returns status
- * code.
- */
+// Entered after the '<' and the first character after that. Returns stat
+// code.
 static char*
 read_element(PInfo pi) {
     struct _attrStack	attrs;
@@ -439,10 +474,9 @@ read_element(PInfo pi) {
     c = *pi->s;
     *end = '\0';
     if ('/' == c) {
-	/* empty element, no attributes and no children */
+	// empty element, no attributes and no children
 	pi->s++;
 	if ('>' != *pi->s) {
-	    /*printf("*** '%s' ***\n", pi->s); */
 	    attr_stack_cleanup(&attrs);
 	    set_error(&pi->err, "invalid format, element not closed", pi->str, pi->s);
 	    return 0;
@@ -480,8 +514,8 @@ read_element(PInfo pi) {
 	    pi->s++;
 	    pi->pcb->add_element(pi, ename, attrs.head, hasChildren);
 	    pi->pcb->end_element(pi, ename);
-
 	    attr_stack_cleanup(&attrs);
+
 	    return 0;
 	case '>':
 	    /* has either children or a value */
@@ -545,6 +579,12 @@ read_element(PInfo pi) {
 	while (!done) {
 	    start = pi->s;
 	    next_non_white(pi);
+	    if (OffSkip == pi->options->skip && start < pi->s && '<' == *pi->s) {
+		c = *pi->s;
+		*pi->s = '\0';
+		pi->pcb->add_text(pi, start, 1);
+		*pi->s = c;
+	    }
 	    c = *pi->s++;
 	    if ('\0' == c) {
 		attr_stack_cleanup(&attrs);
@@ -765,9 +805,11 @@ read_text(PInfo pi) {
     }
     *b = '\0';
     if (0 != alloc_buf) {
+	fix_newlines(alloc_buf);
 	pi->pcb->add_text(pi, alloc_buf, ('/' == *(pi->s + 1)));
 	xfree(alloc_buf);
     } else {
+	fix_newlines(buf);
 	pi->pcb->add_text(pi, buf, ('/' == *(pi->s + 1)));
     }
 }
@@ -834,9 +876,11 @@ read_reduced_text(PInfo pi) {
     }
     *b = '\0';
     if (0 != alloc_buf) {
+	fix_newlines(alloc_buf);
 	pi->pcb->add_text(pi, alloc_buf, ('/' == *(pi->s + 1)));
 	xfree(alloc_buf);
     } else {
+	fix_newlines(buf);
 	pi->pcb->add_text(pi, buf, ('/' == *(pi->s + 1)));
     }
 }
@@ -895,6 +939,7 @@ read_cdata(PInfo pi) {
     *end = '\0';
     pi->s = end + 3;
     if (0 != pi->pcb->add_cdata) {
+	fix_newlines(start);
 	pi->pcb->add_cdata(pi, start, end - start);
     }
 }
@@ -1001,11 +1046,13 @@ read_coded_chars(PInfo pi, char *text) {
     char	*b, buf[32];
     char	*end = buf + sizeof(buf) - 1;
     char	*s;
+    long	blen = 0;
 
     for (b = buf, s = pi->s; b < end; b++, s++) {
 	*b = *s;
 	if (';' == *s) {
 	    *(b + 1) = '\0';
+	    blen = b - buf;
 	    s++;
 	    break;
 	}
@@ -1026,18 +1073,9 @@ read_coded_chars(PInfo pi, char *text) {
 	} else {
 	    if (u <= 0x000000000000007FULL) {
 		*text++ = (char)u;
-#if HAS_PRIVATE_ENCODING
-	    } else if (ox_utf8_encoding == pi->options->rb_enc ||
-		       0 == strcasecmp(rb_str_ptr(rb_String(ox_utf8_encoding)), rb_str_ptr(rb_String(pi->options->rb_enc)))) {
-#else
 	    } else if (ox_utf8_encoding == pi->options->rb_enc) {
-#endif
 		text = ox_ucs_to_utf8_chars(text, u);
-#if HAS_PRIVATE_ENCODING
-	    } else if (Qnil == pi->options->rb_enc) {
-#else
 	    } else if (0 == pi->options->rb_enc) {
-#endif
 		pi->options->rb_enc = ox_utf8_encoding;
 		text = ox_ucs_to_utf8_chars(text, u);
 	    } else if (TolerantEffort == pi->options->effort) {
@@ -1048,30 +1086,20 @@ read_coded_chars(PInfo pi, char *text) {
 	    } else {
 		/*set_error(&pi->err, "Invalid encoding, need UTF-8 or UTF-16 encoding to parse &#nnnn; character sequences.", pi->str, pi->s); */
 		set_error(&pi->err, "Invalid encoding, need UTF-8 encoding to parse &#nnnn; character sequences.", pi->str, pi->s);
-		return 0;
+		return NULL;
 	    }
 	    pi->s = s;
 	}
-    } else if (0 == strcasecmp(buf, "nbsp;")) {
-	pi->s = s;
-	*text++ = ' ';
-    } else if (0 == strcasecmp(buf, "lt;")) {
-	pi->s = s;
-	*text++ = '<';
-    } else if (0 == strcasecmp(buf, "gt;")) {
-	pi->s = s;
-	*text++ = '>';
-    } else if (0 == strcasecmp(buf, "amp;")) {
-	pi->s = s;
-	*text++ = '&';
-    } else if (0 == strcasecmp(buf, "quot;")) {
-	pi->s = s;
-	*text++ = '"';
-    } else if (0 == strcasecmp(buf, "apos;")) {
-	pi->s = s;
-	*text++ = '\'';
     } else {
-	*text++ = '&';
+	char	*t2;
+
+	buf[blen] = '\0';
+	if (NULL == (t2 = ox_entity_lookup(text, buf))) {
+	    *text++ = '&';
+	} else {
+	    text = t2;
+	    pi->s = s;
+	}
     }
     return text;
 }
@@ -1113,19 +1141,10 @@ collapse_special(PInfo pi, char *str) {
 		}
 		if (u <= 0x000000000000007FULL) {
 		    *b++ = (char)u;
-#if HAS_PRIVATE_ENCODING
-		} else if (ox_utf8_encoding == pi->options->rb_enc ||
-			   0 == strcasecmp(rb_str_ptr(rb_String(ox_utf8_encoding)), rb_str_ptr(rb_String(pi->options->rb_enc)))) {
-#else
 		} else if (ox_utf8_encoding == pi->options->rb_enc) {
-#endif
 		    b = ox_ucs_to_utf8_chars(b, u);
 		    /* TBD support UTF-16 */
-#if HAS_PRIVATE_ENCODING
-		} else if (Qnil == pi->options->rb_enc) {
-#else
 		} else if (0 == pi->options->rb_enc) {
-#endif
 		    pi->options->rb_enc = ox_utf8_encoding;
 		    b = ox_ucs_to_utf8_chars(b, u);
 		} else {
@@ -1154,16 +1173,30 @@ collapse_special(PInfo pi, char *str) {
 		    *b++ = '&';
 		    continue;
 		} else {
-		    c = '?';
+		    char	key[16];
+		    char	*k = key;
+		    char	*kend = key + sizeof(key) - 1;
+
+		    *k++ = *s;
 		    while (';' != *s++) {
 			if ('\0' == *s) {
 			    set_error(&pi->err, "Invalid format, special character does not end with a semicolon", pi->str, pi->s);
 			    return EDOM;
 			}
+			if (kend <= k) {
+			    k = key;
+			    break;
+			}
+			*k++ = *s;
 		    }
-		    s++;
-		    set_error(&pi->err, "Invalid format, invalid special character sequence", pi->str, pi->s);
-		    return 0;
+		    k--;
+		    *k = '\0';
+		    if ('\0' == *key || NULL == (b = ox_entity_lookup(b, key))) {
+			set_error(&pi->err, "Invalid format, invalid special character sequence", pi->str, pi->s);
+			c = '?';
+			return 0;
+		    }
+		    continue;
 		}
 		*b++ = (char)c;
 	    }
