@@ -8,7 +8,7 @@
 #include <stdio.h>
 #include <strings.h>
 #include <sys/types.h>
-#if NEEDS_UIO
+#if HAVE_SYS_UIO_H
 #include <sys/uio.h>
 #endif
 #include <unistd.h>
@@ -43,7 +43,7 @@ parse_double_time(const char *text) {
     for (; text - dot <= 9; text++) {
         v2 *= 10;
     }
-#if HAS_NANO_TIME
+#if HAVE_RB_TIME_NANO_NEW
     return rb_time_nano_new(v, v2);
 #else
     return rb_time_new(v, v2 / 1000);
@@ -103,7 +103,7 @@ parse_xsd_time(const char *text) {
     tm.tm_hour = (int)cargs[3];
     tm.tm_min = (int)cargs[4];
     tm.tm_sec = (int)cargs[5];
-#if HAS_NANO_TIME
+#if HAVE_RB_TIME_NANO_NEW
     return rb_time_nano_new(mktime(&tm), cargs[6]);
 #else
     return rb_time_new(mktime(&tm), cargs[6] / 1000);
@@ -136,15 +136,9 @@ sax_value_as_s(VALUE self) {
 	break;
     }
     rs = rb_str_new2(dr->buf.str);
-#if HAS_ENCODING_SUPPORT
     if (0 != dr->encoding) {
 	rb_enc_associate(rs, dr->encoding);
     }
-#elif HAS_PRIVATE_ENCODING
-    if (Qnil != dr->encoding) {
-	rb_funcall(rs, ox_force_encoding_id, 1, dr->encoding);
-    }
-#endif
     return rs;
 }
 
@@ -159,7 +153,7 @@ sax_value_as_sym(VALUE self) {
     if ('\0' == *dr->buf.str) {
 	return Qnil;
     }
-    return str2sym(dr, dr->buf.str, 0);
+    return str2sym(dr, dr->buf.str, strlen(dr->buf.str), 0);
 }
 
 /* call-seq: as_f()
