@@ -3,10 +3,12 @@
  * All rights reserved.
  */
 
-#ifndef __OX_SAX_BUF_H__
-#define __OX_SAX_BUF_H__
+#ifndef OX_SAX_BUF_H
+#define OX_SAX_BUF_H
 
-typedef struct _Buf {
+#include <stdio.h>
+
+typedef struct _buf {
     char	base[0x00001000];
     char	*head;
     char	*end;
@@ -14,26 +16,26 @@ typedef struct _Buf {
     char	*read_end;      /* one past last character read */
     char       	*pro;           /* protection start, buffer can not slide past this point */
     char        *str;           /* start of current string being read */
-    int		pos;
-    int		line;
-    int		col;
-    int		pro_pos;
-    int		pro_line;
-    int		pro_col;
-    int		(*read_func)(struct _Buf *buf);
+    off_t	pos;
+    off_t	line;
+    off_t	col;
+    off_t	pro_pos;
+    off_t	pro_line;
+    off_t	pro_col;
+    int		(*read_func)(struct _buf *buf);
     union {
         int     	fd;
         VALUE   	io;
 	const char	*str;
     } in;
-    struct _SaxDrive	*dr;
+    struct _saxDrive	*dr;
 } *Buf;
 
-typedef struct _CheckPt {
-    int		pro_dif;
-    int		pos;
-    int		line;
-    int		col;
+typedef struct _checkPt {
+    off_t	pro_dif;
+    off_t	pos;
+    off_t	line;
+    off_t	col;
     char	c;
 } *CheckPt;
 
@@ -219,4 +221,4 @@ buf_collapse_white(char *str) {
     *back = '\0';
 }
 
-#endif /* __OX_SAX_BUF_H__ */
+#endif /* OX_SAX_BUF_H */
